/**
 ** A Virtual Host Monitor
 **
 ** Copyright 2009 Gary Wuertz gary@issiweb.com
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <string.h>

#include "config.h"

using namespace std;
/**
 * Constructor
 */
Config::Config(char const* path)
{
   this->read(strlen(path)>0? string(path) : this->filename());
   this->load(this->get("default").c_str());
}
/**
 * Implement \n in the string
 */
string Config::cntrl(string s)
{
   for (unsigned i=0;(i+1)<s.length();i++)
      if (s[i]=='\\' && s[i+1]=='n') {
         s[i] = '\n';
         s.erase(i+1,1);
         }
   return s;
}
/**
 * Dump the configuration
 */
void Config::dump()
{
   map<string,string>::iterator iter;
   for(iter = this->content.begin(); iter != this->content.end(); ++iter )
      cout << iter->first << " = " << iter->second << endl;
}
/**
 * Get the application configuration
 */
string Config::filename()
{
   string home = getenv("HOME");
   return home+"/.topvhost";
}
/**
 * Fetch a value from current section with fallback to main section
 */
string Config::get(char const* name)
{
   string v = this->content[this->section + "/" + name];
   if (v.length()>0) return v;
   v = "/";
   return this->content[v+name];
}
/**
 * Get all settings in a section
 */
map<string,string>* Config::getSection(char const* name)
{
   map<string,string>* result = new map<string,string>;
   map<string,string>::iterator iter;
   int len,len2;

   if (*name=='/') name++;
   len = len2 = strlen(name);
   len2 += len? 2 : 1;
   for(iter=this->content.begin();iter!=this->content.end();++iter) {
      string key = iter->first.substr(1,len);
      if (key.compare(0,len,name)==0)
         (*result)[iter->first.substr(len2)]=iter->second;
      }
   return result;
}
/**
 * Set the section
 */
void Config::load(char const* section)
{
   this->section = "/"+this->trim(section);
}
/**
 * Read configuration from file
 */
void Config::read(string const& configFile)
{
   ifstream file(configFile.c_str());
   string line;
   string name;
   string value;
   string inSection;
   int posEqual;

   while (getline(file,line)) {
      if (!line.length()||line[0]=='#'||line[0]==';') continue;
      if (line[0] == '[') {
         inSection='/'+this->trim(line.substr(1,line.find(']')-1));
         continue;
         }
      posEqual=line.find('=');
      name  = this->trim(line.substr(0,posEqual));
      value = this->trim(line.substr(posEqual+1));
      this->content[inSection+"/"+name]=value;
      }
}
/**
 * Fetch a value
 */
void Config::set(char const* name, char const* value)
{
   this->content[this->section + "/" + name] = value;
}
/**
 * Trim a string
 */
string Config::trim(string const& source, char const* delims)
{
   string result(source);
   string::size_type index = result.find_last_not_of(delims);
   if(index != string::npos)
      result.erase(++index);
   index = result.find_first_not_of(delims);
   if(index != string::npos)
      result.erase(0, index);
   else  result.erase();
   return result;
}
